#coding: utf-8
import hashlib
import json
import os
import sys
import threading
import time

import wx
import yaml
from pubsub import pub

from lib.socket.ClientWebSocket import ClientWebSocket
from ui import globalParams


class HomeArea():
    def __init__(self, frame):
        self.frame = frame
        self.wsUrl = "ws://api-test.vandyo.com/mirror/websocket/"                     # webosocket 连接地址
        self.devCode = "868142043390091"                                              # 设备id
        self.ws = None
        self.mainPanel = None
        self.heartBeatTime = 30                                                       # 心跳时间
        self.isHeartBeat = 1                                                          # 是否开启心跳 0：不开启   1：开启
        self.ignt = 1                                                                 # 点熄火状态  0：熄火  1：点火

        self.wsUrlText = None                                                        # webSocket 连接地址文本框
        self.devCodeText = None                                                      # 设备号文本框
        self.heartBeatTimeText = None                                                # 心跳时间文本框
        self.heartBeatCombox = None                                                  # 心跳开关设置

        self.logTextCtr = None                                                       # 日志显示框

        with open("config/config.yaml", 'r', encoding="utf-8") as fi:
            fi_data = fi.read()
            data = yaml.load(fi_data,Loader=yaml.FullLoader)
            self.devCode = str(data["cameraParam"]["devCode"])



    def setWsUrl(self,data):
        self.wsUrl = data
    def setDevCode(self,data):
        self.devCode = data

    def getDevCodeText(self):
        return self.devCodeText

    #################################################
    #       创建一个顶级pannel
    #################################################
    def create(self):
        self.mainPanel = wx.Panel(self.frame)
        boxSizer = wx.BoxSizer(wx.VERTICAL)
        topPanel = self.getTopPanel()

        bottomPanel = self.getBottomPanel()
        boxSizer.Add(topPanel,1,flag=wx.EXPAND | wx.ALL)
        boxSizer.Add(bottomPanel,2, flag=wx.EXPAND | wx.TOP , border=5)
        self.mainPanel.SetSizer(boxSizer)

        pub.subscribe(self.processRevCmd, "msgRev")
        return self.mainPanel

    def getTopPanel(self):
        topPanel = wx.Panel(self.mainPanel)
        # topPanel.SetBackgroundColour("PINK")
        wx.StaticText(topPanel, label='连接地址：', pos=(10, 10))
        self.wsUrlText = wx.TextCtrl(topPanel, pos=(70, 5), size=wx.Size(355, -1), value=self.wsUrl)
        wx.StaticText(topPanel, label='设备 号：', pos=(430, 10))
        self.devCodeText = wx.TextCtrl(topPanel, pos=(485, 5), size=wx.Size(120, -1), value=self.devCode)
        setDevCodeButton = wx.Button(topPanel, label="设置设备号", pos=(610, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.setDevCode(evt),
                        setDevCodeButton)
        wx.StaticText(topPanel, label='心跳时间（秒）:', pos=(10, 40))
        self.heartBeatTimeText = wx.TextCtrl(topPanel, pos=(110,35), size=wx.Size(60, -1), value=str(self.heartBeatTime))
        wx.StaticText(topPanel, label='是否开启心跳:', pos=(180, 40))
        self.heartBeatCombox = wx.ComboBox(topPanel, pos=(270, 35), choices=['开', '关'], value="关",
                                           size=wx.Size(50, -1))
        connectButton = wx.Button(topPanel, label="连接设备", pos=(610, 35))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.connect(evt),
                        connectButton)
        disConnectButton = wx.Button(topPanel, label="断开连接", pos=(690, 35))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.disConnect(evt),
                        disConnectButton)
        wx.StaticText(topPanel, label='状态：',pos=(780, 40))
        self.connectStatusText = wx.TextCtrl(topPanel, pos=(820, 35), size=wx.Size(80, -1), value="未连接",style=wx.TE_READONLY)
        self.connectStatusText.SetForegroundColour(wx.RED)

        return topPanel

    #################################################
    #       获取底部pannel的内容
    #################################################
    def getBottomPanel(self):
        bottomPanel = wx.Panel(self.mainPanel)
        boxSizer_bottom = wx.BoxSizer(wx.HORIZONTAL)
        logArea = wx.Panel(bottomPanel)
        logStaticText = wx.StaticText(logArea, label='日志：')
        self.logTextCtr = wx.TextCtrl(logArea, size=wx.Size(40, -1),style=wx.TE_MULTILINE)
        boxSizer_logArea = wx.BoxSizer(wx.VERTICAL)
        boxSizer_logArea.Add(logStaticText, 2, flag=wx.EXPAND | wx.TOP ,border=10)
        boxSizer_logArea.Add(self.logTextCtr, 100, flag=wx.EXPAND | wx.ALL)
        logArea.SetSizer(boxSizer_logArea)

        ctrArea = wx.Panel(bottomPanel)
        ctrText = wx.StaticText(ctrArea, label='控制：')
        ctrPanel = wx.Panel(ctrArea, style=wx.BORDER_SIMPLE)
        clearLogButton = wx.Button(ctrPanel, label="清空日志", pos=(5, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.clearLog(evt),
                        clearLogButton)
        clearLogButton = wx.Button(ctrPanel, label="关闭心跳", pos=(85, 5))
        self.frame.Bind(wx.EVT_BUTTON, lambda evt: self.stopHeartBeat(evt),
                        clearLogButton)
        boxSizer_ctrArea = wx.BoxSizer(wx.VERTICAL)
        boxSizer_ctrArea.Add(ctrText,2, flag=wx.EXPAND | wx.TOP ,border=10)
        boxSizer_ctrArea.Add(ctrPanel,100, flag=wx.EXPAND | wx.ALL)
        ctrArea.SetSizer(boxSizer_ctrArea)

        boxSizer_bottom.Add(logArea,60,flag=wx.EXPAND | wx.ALL)
        boxSizer_bottom.Add(ctrArea, 40, flag=wx.EXPAND | wx.LEFT,border=10)
        bottomPanel.SetSizer(boxSizer_bottom)

        return  bottomPanel

    #################################################
    #       设置设备号
    #################################################
    def setDevCode(self,evt):
        globalParams.camera_1.getDevIdText().SetValue(self.devCodeText.GetValue()[3:])
        globalParams.camera_2.getDevIdText().SetValue(self.devCodeText.GetValue()[3:])
        globalParams.camera_3.getDevIdText().SetValue(self.devCodeText.GetValue()[3:])

    #################################################
    #       连接设备
    #################################################
    def connect(self,evt):
        self.setLogRedirect()
        self.ws = ClientWebSocket()
        self.ws.setWsUrl(self.wsUrlText.GetValue())
        self.ws.setWsCode(self.devCodeText.GetValue())
        threadObj = threading.Thread(target=self.doConnect, args=())
        threadObj.start()
        if self.heartBeatCombox.GetValue() == "开":
            self.isHeartBeat = 1
            self.startHeartBeat()
        self.connectStatusText.SetValue("已连接")
    def doConnect(self):
        self.ws.connect()


    #################################################
    #       断开连接
    #################################################
    def disConnect(self,evt):
        self.setLogRedirect()
        self.stopHeartBeat(None)
        self.ws.close()
        self.connectStatusText.SetValue("未连接")

        # self.stopHeartBeat(None)
        # self.ws.close()
        # self.connectStatusText.SetValue("未连接")
        # curTime = self.getCurTime()
        # self.logTextCtr.WriteText(curTime + "websocket连接已经断开")

    #################################################
    #       清空日志
    #################################################
    def clearLog(self,evt):
        self.setLogRedirect()
        self.logTextCtr.SetValue("")


    #################################################
    #       启动发送心跳服务
    #################################################
    def startHeartBeat(self):
        self.setLogRedirect()
        threadObj = threading.Thread(target=self.sendHeartBeatService, args=())
        threadObj.start()
    #################################################
    #       发送心跳服务
    #################################################
    def sendHeartBeatService(self):
        self.setLogRedirect()
        while self.isHeartBeat == 1:
            time.sleep(int(self.heartBeatTimeText.GetValue()))
            if self.isHeartBeat == 1:
                timeCur = int(time.time() * 1000)
                skeyOrg = str(timeCur) + self.devCodeText.GetValue()
                m = hashlib.md5()
                m.update(skeyOrg.encode("utf8"))
                skey = m.hexdigest()
                msg = {}
                msg["data"] = {}
                msg["data"]["ignt"] = self.ignt
                msg["skey"] = skey
                msg["type"] = "9"         # 心跳指令上报
                msg = json.dumps(msg)
                self.ws.sendMsg(msg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "发送心跳：" + msg + "\n")

    #################################################
    #       停止发送心跳服务
    #################################################
    def stopHeartBeat(self,evt):
        self.setLogRedirect()
        self.isHeartBeat = 0
        curTime = self.getCurTime()
        self.logTextCtr.WriteText(curTime + "停止了心跳服务！\n")

    ####################################################
    #      获取当前时间
    ####################################################
    def getCurTime(self):
        timeCur = int(time.time() * 1000)
        timeArray = time.localtime(timeCur / 1000)
        timeCur = time.strftime("%Y-%m-%d %H:%M:%S", timeArray)
        timeCur = "[" + timeCur + "]  "
        return timeCur


    ####################################################
    #      处理服务端下发的指令
    ####################################################
    def processRevCmd(self,msg):
        self.setLogRedirect()
        curTime = self.getCurTime()
        self.logTextCtr.WriteText(curTime + "收到消息：" + msg + "\n")
        msgJson = json.loads(msg)
        msgType = msgJson["type"]
        rSkey = msgJson["skey"]
        replyMsg = {}
        replyMsg["skey"] = rSkey
        if msgType == "1":                                           # 请求直播
            self.processStream(msgJson, replyMsg)
        elif msgType == "3":                                         # 视频抓拍
            self.processVideoCap(msgJson, replyMsg)
        elif msgType == "4":                                         # 图片抓拍
            self.processPicCap(msgJson, replyMsg)
        elif msgType == "5":                                         # 视频回看
            self.processSeeHistoryVideo(msgJson,replyMsg)
        elif msgType == "7":                                         # 紧急救援
            pass
    ####################################################
    #      处理直播请求
    ####################################################
    def processStream(self,msgJson,replyMsg):
        cam = msgJson["data"]["cam"]
        if cam == 1:
            if globalParams.camera_1.getPushObj() == None:
                globalParams.camera_1.doConnect()
                globalParams.camera_1.startPush(None)
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")
            else:
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "消息： 摄像头1连接中！！！\n")
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")
        elif cam == 2:
            if globalParams.camera_2.getPushObj() == None:
                globalParams.camera_2.doConnect()
                globalParams.camera_2.startPush(None)
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")
            else:
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "消息： 摄像头2连接中！！！\n")
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")
        elif cam == 3:
            if globalParams.camera_3.getPushObj() == None:
                globalParams.camera_3.doConnect()
                globalParams.camera_3.startPush(None)
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")
            else:
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "消息： 摄像头3连接中！！！\n")
                replyMsg["type"] = "0"
                replyMsg = json.dumps(replyMsg)
                self.ws.sendMsg(replyMsg)
                curTime = self.getCurTime()
                self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")

    ####################################################
    #      处理视频抓拍
    ####################################################
    def processVideoCap(self,msgJson,replyMsg):
        cam = msgJson["data"]["cam"]
        if cam == 1:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_1.videoCapture(cam,replyMsg,self.ws)
        elif cam == 2:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_2.videoCapture(cam,replyMsg,self.ws)
        elif cam == 3:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_3.videoCapture(cam,replyMsg,self.ws)

    ####################################################
    #      处理图片抓拍
    ####################################################
    def processPicCap(self,msgJson,replyMsg):
        cam = msgJson["data"]["cam"]
        if cam == 1:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_1.picCapture(cam,replyMsg,self.ws)
        elif cam == 2:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_2.picCapture(cam,replyMsg,self.ws)
        elif cam == 3:
            replyMsg["type"] = "0"
            replyMsg = json.dumps(replyMsg)
            globalParams.camera_3.picCapture(cam,replyMsg,self.ws)

    ####################################################
    #      处理视频回看
    ####################################################
    def processSeeHistoryVideo(self,msgJson,replyMsg):
        cam = msgJson["data"]["cam"]
        fiList = self.getFileList(cam)
        replyMsg["code"] = "0"
        replyMsg["type"] = "5"
        replyMsg["data"] = fiList
        replyMsg = json.dumps(replyMsg)
        self.ws.sendMsg(replyMsg)
        curTime = self.getCurTime()
        self.logTextCtr.WriteText(curTime + "回复消息：" + replyMsg + "\n")


    ####################################################
    #      日志重定向到日志框
    ####################################################
    def setLogRedirect(self):
        if globalParams.isLogRedirect == 1:
            sys.stdout = self.logTextCtr
            sys.stderr = self.logTextCtr

    ####################################################
    #      获取视频文件列表
    ####################################################
    def getFileList(self,cam):
        dirPath = "D:/project/python/mirrorSimulator/"
        dirPath = dirPath + "SDCard/video/CAM_" + str(cam) + "/"
        fileList = {}
        fileList["cam"] = str(cam)
        fileList["files"] = []
        file_names = os.listdir(dirPath)
        for fn in file_names:
            fileInfo = {}
            timeLen = 12
            fsize = os.path.getsize(dirPath + fn)
            fsize = int(fsize / 1024)
            fiSplit = fn.split("_")
            fTime = fiSplit[3] + "-" + fiSplit[4][:-4]
            fTime = fTime[:4] + "-" + fTime[4:6] + "-" + fTime[6:8] + " " + fTime[9:11] + ":" + fTime[11:13] + ":" + fTime[13:]
            timeArray = time.strptime(fTime, "%Y-%m-%d %H:%M:%S")
            timeStamp = int(time.mktime(timeArray))
            fileInfo["fname"] = fn
            fileInfo["fsize"] = str(fsize)
            fileInfo["ftime"] = timeLen
            fileInfo["fdate"] = timeStamp
            fileInfo["fileaddress"] = dirPath + fn
            fileList["files"].append(fileInfo)
        return fileList


if __name__ == "__main__":
    dirPath = "D:/project/python/mirrorSimulator/"
    dirPath = dirPath + "SDCard/video/CAM_1/"
    fileList = {}
    fileList["cam"] = 1
    fileList["files"] = []
    file_names = os.listdir(dirPath)
    for fn in file_names:
        fileInfo = {}
        timeLen = 12
        fsize = os.path.getsize(dirPath + fn)
        fsize = int(fsize / 1024)
        fiSplit = fn.split("_")
        fTime = fiSplit[3] + "-" + fiSplit[4][:-4]
        fileInfo["fname"] = fn
        fileInfo["fsize"] = fsize
        fileInfo["ftime"] = timeLen
        fileInfo["fdate"] = fTime
        fileList["files"].append(fileInfo)
    print(fileList)


